package com.softslate.commerce.businessobjects.core;

import java.util.Map;
import java.util.Properties;

import javax.inject.Provider;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpSession;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import com.google.common.eventbus.EventBus;
import com.google.inject.Inject;
import com.google.inject.name.Names;
import com.google.inject.servlet.ServletModule;
import com.google.inject.servlet.ServletScopes;
import com.softslate.commerce.daos.core.DAOFactory;

public class GuiceBaseModule extends ServletModule {

	static Log log = LogFactory.getLog(GuiceBaseModule.class);

	private Properties appComponents = null;
	private Properties appComponentsDefaults = null;
	private Map singletons = null;
	private EventBus eventBus = new EventBus();
	private boolean noServlet = false;

	public GuiceBaseModule(Properties appComponents, Properties appComponentsDefaults, Map singletons, EventBus eventBus) {
		this.appComponents = appComponents;
		this.appComponentsDefaults = appComponentsDefaults;
		this.singletons = singletons;
		this.eventBus = eventBus;
	}

	public GuiceBaseModule(Properties appComponents, Properties appComponentsDefaults, Map singletons, boolean noServlet) {
		this.appComponents = appComponents;
		this.appComponentsDefaults = appComponentsDefaults;
		this.singletons = singletons;
		this.noServlet = noServlet;
	}

	@Override
	protected void configureServlets() {
		bindSingletons();
		bindFromComponentsFiles();
	}

	protected void bindSingletons() {
		bind(DAOFactory.class).toInstance((DAOFactory) singletons.get("daoFactory"));
		bind(BusinessObjectFactory.class).toInstance((BusinessObjectFactory) singletons.get("businessObjectFactory"));
		bind(Settings.class).toInstance((Settings) singletons.get("settings"));
		bind(Properties.class).annotatedWith(Names.named("appComponents")).toInstance(
				(Properties) singletons.get("appComponents"));
		bind(Properties.class).annotatedWith(Names.named("appSettings")).toInstance(
				(Properties) singletons.get("appSettings"));
		bind(EventBus.class).toInstance(eventBus);
	}

	protected void bindFromComponentsFiles() {
		for (String componentKey : appComponents.stringPropertyNames()) {
			if (!componentKey.equals("settingsImplementer") && !componentKey.equals("businessObjectFactoryImplementer")
					&& !componentKey.equals("daoFactoryImplementer")) {
				String componentValue = appComponents.getProperty(componentKey);
				String componentDefaultValue = appComponentsDefaults.getProperty(componentKey);
				log.debug("componentKey " + componentKey + " componentValue: " + componentValue
						+ " componentDefaultValue: " + componentDefaultValue);
				Class concreteClass = null;
				try {
					concreteClass = Class.forName(componentValue);
					Class boundClass = null;
					if (componentDefaultValue == null) {
						boundClass = interfaceFromComponent(componentKey, componentValue);
					} else {
						boundClass = interfaceFromComponent(componentKey, componentDefaultValue);
					}
					if (boundClass != null && boundClass.isAssignableFrom(concreteClass)) {
						log.info("Binding " + boundClass.getName() + " to: " + concreteClass.getName());
						if (componentKey.equals("userImplementer") && !noServlet) {
							bindUserObject();
						} else if (componentKey.equals("businessObjectFactoryImplementer") && !noServlet) {
							bindBusinessObjectFactory();
						} else if (componentKey.equals("settingsImplementer") && !noServlet) {
							bindSettingsObject();
						} else if (boundClass.equals(concreteClass)) {
							bind(concreteClass);
						} else {
							bind(boundClass).to(concreteClass);
						}
					}
				} catch (ClassNotFoundException cnfe) {
					log.warn("Skipping Guice binding for concrete class as it can't be found "
							+ "(normal for Enterprise Edition classes when running the Community Edition): "
							+ componentValue);
				}
			}
		}
	}

	protected Class interfaceFromComponent(String componentKey, String componentValue) {
		Class interfaceClassDef = null;
		String interfaceFullName = null;
		String packageName = componentValue.substring(0, componentValue.lastIndexOf("."));
		String interfaceName = componentKey.substring(0, 1).toUpperCase()
				+ componentKey.substring(1).replaceFirst("Implementer", "");
		interfaceName = interfaceName.replace("Dao", "DAO");
		interfaceName = interfaceName.replace("Sku", "SKU");
		interfaceName = interfaceName.replace("CreSecure", "CRESecure");
		interfaceFullName = packageName + "." + interfaceName;
		try {
			interfaceClassDef = Class.forName(interfaceFullName);
		} catch (ClassNotFoundException cnfe) {
			log.info("Can't derive interface from default component (will try from concrete class): " + componentKey
					+ "; interfaceFullName attempted: " + interfaceFullName + "; componentDefaultValue: "
					+ componentValue);
		}
		if (interfaceClassDef == null) {
			try {
				Class defaultConcreteClass = Class.forName(componentValue);
				interfaceClassDef = interfaceFromConcreteClass(defaultConcreteClass);
			} catch (ClassNotFoundException cnfe) {
				log.warn("Skipping Guice binding for concrete class as it can't be found "
						+ "(normal for Enterprise Edition classes when running the Community Edition): "
						+ componentValue);
			}

		}
		return interfaceClassDef;
	}

	protected Class interfaceFromConcreteClass(Class concreteClass) {
		Class interfaceClassDef = null;
		Class[] interfaces = concreteClass.getInterfaces();
		if (interfaces.length > 0) {
			boolean interfaceFoundInSamePackage = false;
			for (int i = 0; i < interfaces.length; i++) {
				Class thisInt = interfaces[i];
				if (thisInt.getPackage().equals(concreteClass.getPackage())) {
					if (!interfaceFoundInSamePackage || concreteClass.getSimpleName().contains(thisInt.getSimpleName())) {
						interfaceClassDef = thisInt;
						interfaceFoundInSamePackage = true;
					}
				}
			}

		}
		return interfaceClassDef;
	}

	protected void bindUserObject() {
		bind(UserProvider.class).in(ServletScopes.SESSION);
		bind(User.class).toProvider(UserProvider.class).in(ServletScopes.SESSION);
	}

	static protected class UserProvider implements Provider<User> {
		@Inject
		HttpSession session;

		public User get() {
			return (User) session.getAttribute("user");
		}
	}

	protected void bindBusinessObjectFactory() {
		bind(BOFProvider.class).in(ServletScopes.REQUEST);
		bind(BusinessObjectFactory.class).toProvider(BOFProvider.class).in(ServletScopes.REQUEST);
	}

	static protected class BOFProvider implements Provider<BusinessObjectFactory> {
		@Inject
		HttpServletRequest request;

		public BusinessObjectFactory get() {
			return (BusinessObjectFactory) request.getAttribute("businessObjectFactory");
		}
	}

	protected void bindSettingsObject() {
		bind(SettingsProvider.class).in(ServletScopes.REQUEST);
		bind(Settings.class).toProvider(SettingsProvider.class).in(ServletScopes.REQUEST);
	}

	static protected class SettingsProvider implements Provider<Settings> {
		@Inject
		HttpServletRequest request;

		public Settings get() {
			return (Settings) request.getAttribute("settings");
		}
	}

}
