package com.softslate.commerce.businessobjects.core;

import java.util.Collection;
import java.util.Locale;
import java.util.Map;

import com.softslate.commerce.businessobjects.administrator.Administrator;
import com.softslate.commerce.businessobjects.customer.Customer;
import com.softslate.commerce.businessobjects.order.Order;

/**
 * Interface representing an end user accessing the system. In particular,
 * either a customer using the user interface to shop for and purchase products
 * or an administrator configuring the store's administrative settings.
 * <p>
 * An instance implementing <code>User</code> is created on the first request
 * of a user's session and is stored along with that session for as long as the
 * user continues to access the application. It is stored as a session attribute
 * named "user".
 * <p>
 * When the application needs to create an instance that implements
 * <code>User</code>,
 * {@link com.softslate.commerce.businessobjects.core.BusinessObjectFactory}
 * finds the name of the Java class to instantiate from the "userImplementer"
 * setting in the <code>appComponents.properties</code> file.
 * <p>
 * The default "userImplementer" is
 * {@link com.softslate.commerce.businessobjects.core.UserBean}.
 * 
 * @author David Tobey
 * 
 * 
 */
public interface User extends BusinessObject {

	/**
	 * Retrieves the <code>Locale</code> in effect for this <code>User</code>.
	 * Useful to assist with the formatting of currencies and other
	 * localizations.
	 * 
	 * @return The <code>Locale</code> in effect for this <code>User</code>.
	 */
	public Locale getLocale();

	/**
	 * Sets the <code>Locale</code> in effect for this <code>User</code>.
	 * This <code>Locale</code> is set in the Struts layer directly after a
	 * <code>User</code> object is created in each user's session.
	 */
	public void setLocale(Locale locale);

	/**
	 * Retrieves the <code>Order</code> this <code>User</code> is currently
	 * placing. This <code>Order</code> object is created when the user adds
	 * the first item to his or her cart. It is destroyed either when the user
	 * completes the order, or when his or her session ends.
	 * 
	 * @return The <code>Order</code> this <code>User</code> is currently
	 *         placing.
	 */
	public Order getOrder();

	/**
	 * Sets the <code>Order</code> this <code>User</code> is currently
	 * placing. This <code>Order</code> object is created when the user adds
	 * the first item to his or her cart. It is destroyed either when the user
	 * completes the order, or when his or her session ends.
	 * 
	 * @param order
	 *            The <code>Order</code> this <code>User</code> is currently
	 *            placing.
	 */
	public void setOrder(Order order);

	/**
	 * Retrieves the <code>Customer</code> object associated with this
	 * <code>User</code>. When a user logs in as a customer, this
	 * <code>Customer</code> object is created to store his or her account
	 * settings and other information.
	 * 
	 * @return The <code>Customer</code> object associated with this
	 *         <code>User</code>, or <code>null</code>, if the user has
	 *         not logged in.
	 */
	public Customer getCustomer();

	/**
	 * Sets the <code>Customer</code> object associated with this
	 * <code>User</code>. When a user logs in as a customer, this
	 * <code>Customer</code> object is created to store his or her account
	 * settings and other information.
	 * 
	 * @param customer
	 *            The <code>Customer</code> object associated with this
	 *            <code>User</code>.
	 */
	public void setCustomer(Customer customer);

	/**
	 * Retrieves a <code>Collection</code> of <code>String</code>s
	 * representing coupon codes entered by this <code>User</code>. When a
	 * user enters a coupon code at any point in the session, the code gets
	 * stored here for retrieval during discount processing.
	 * 
	 * @return A <code>Collection</code> of <code>String</code>s
	 *         representing coupon codes entered by this <code>User</code>.
	 */
	public Collection getEnteredCouponCodes();

	/**
	 * Sets the <code>Collection</code> of coupon codes entered by this
	 * <code>User</code>. When a user enters a valid coupon code, the code
	 * gets stored here for later retrieval during discount processing.
	 * 
	 * @param enteredCouponCodes
	 *            A <code>Collection</code> of <code>String</code>s
	 *            representing coupon codes entered by this <code>User</code>.
	 */
	public void setEnteredCouponCodes(Collection enteredCouponCodes);

	/**
	 * Retrieves the <code>Administrator</code> object associated with this
	 * <code>User</code>. When a user logs in as an administrator, this
	 * <code>Administrator</code> object is created to store his or her
	 * account settings and other information.
	 * 
	 * @return The <code>Administrator</code> object associated with this
	 *         <code>User</code>, or <code>null</code>, if the user has
	 *         not logged in.
	 */
	public Administrator getAdministrator();

	/**
	 * Sets the <code>Administrator</code> object associated with this
	 * <code>User</code>. When a user logs in as an administrator, this
	 * <code>Administrator</code> object is created to store his or her
	 * account settings and other information.
	 * 
	 * @param administrator
	 *            The <code>Administrator</code> object associated with this
	 *            <code>User</code>.
	 */
	public void setAdministrator(Administrator administrator);

	/**
	 * Returns a <code>Map</code> of any custom objects saved for this
	 * <code>User</code>. Custom developers may place objects in the map
	 * which need to be stored with the user throughout his session.
	 * 
	 * @return A <code>Map</code> of custom objects saved for this
	 *         <code>User</code>.
	 */
	public Map getCustomMap();

	/**
	 * Sets the <code>Map</code> of custom objects for this <code>User</code>.
	 * Custom developers may place objects in the map which need to be stored
	 * with the user throughout his session.
	 * 
	 * @param data
	 *            A <code>Map</code> of custom objects saved for this
	 *            <code>User</code>
	 */
	public void setCustomMap(Map data);
}
