package com.softslate.commerce.businessobjects.payment;

import java.util.HashMap;
import java.util.LinkedHashSet;
import java.util.Map;

import lp.order.LPOrderFactory;
import lp.order.LPOrderPart;
import lp.txn.JLinkPointTransaction;

import org.apache.commons.beanutils.BeanUtils;
import org.apache.commons.lang.StringUtils;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import com.softslate.commerce.businessobjects.order.Order;
import com.softslate.commerce.businessobjects.payment.BasePaymentProcessor;
import com.softslate.commerce.businessobjects.payment.PaymentProcessor;
import com.softslate.commerce.businessobjects.payment.Payment;
import com.softslate.commerce.daos.payment.PaymentDAO;

/**
 * Processes a payment in the store for a given user's cart. This implementation
 * processes a request through the First Data Global Gateway.
 * 
 * @author David Tobey
 * @see com.softslate.commerce.daos.payment.PaymentDAO
 */
public class FirstDataProcessor extends BasePaymentProcessor implements
		PaymentProcessor {

	static Log log = LogFactory.getLog(FirstDataProcessor.class);

	public Map process(Map parameters) throws Exception {
		log.info("Processing payment.");

		Order order = getUser().getOrder();
		parameters.put("order", order);

		return processTransaction(parameters);
	}

	public Map processTransaction(Map parameters) throws Exception {

		// Set up the connection
		if (log.isInfoEnabled())
			log.info("Initializing transaction.");
		JLinkPointTransaction txn = initializeTransaction(parameters);

		// Build the request
		if (log.isInfoEnabled())
			log.info("Building XML request");
		String xmlRequest = getOrderXML(parameters);

		// Send the request
		String responseData = "";
		try {
			responseData = txn.send(xmlRequest);
		} catch (Exception e) {
			log.error("Exception sending First Data transaction: ", e);
			responseData = "<r_error>" + e.toString() + "</r_error>";
		}

		// process the response
		return processResponse(responseData, parameters);
	}

	public JLinkPointTransaction initializeTransaction(Map parameters) {

		JLinkPointTransaction txn = new JLinkPointTransaction();

		String firstDataClientCertificatePath = (String) parameters
				.get("certPath");
		String firstDataClientCertificatePassword = (String) getSettings()
				.getValue("firstDataClientCertificatePassword");
		String firstDataAPIHostName = (String) getSettings().getValue(
				"firstDataAPIHostName");
		String firstDataAPIPort = (String) getSettings().getValue(
				"firstDataAPIPort");

		if (log.isInfoEnabled())
			log.info("Will be connecting to: " + firstDataAPIHostName
					+ ", port " + firstDataAPIPort
					+ ", using the certificate file at "
					+ firstDataClientCertificatePath);

		txn.setClientCertificatePath(firstDataClientCertificatePath);
		txn.setPassword(firstDataClientCertificatePassword);
		txn.setHost(firstDataAPIHostName);
		txn.setPort(Integer.parseInt(firstDataAPIPort));
		return txn;
	}

	public String getOrderXML(Map parameters) {

		String firstDataTransactionType = (String) getSettings().getValue(
				"firstDataTransactionType");
		String firstDataTestMode = (String) getSettings().getValue(
				"firstDataTestMode");
		String firstDataStoreNumber = (String) getSettings().getValue(
				"firstDataStoreNumber");
		Order userOrder = (Order) parameters.get("order");

		// Create an empty order
		LPOrderPart order = LPOrderFactory.createOrderPart("order");

		// Create a part we will use to build the order
		LPOrderPart op = LPOrderFactory.createOrderPart();

		// Transaction type (PREAUTH or SALE) and test mode
		op.put("ordertype", escapeXML(firstDataTransactionType));
		op.put("result", escapeXML(firstDataTestMode));
		parameters.put("transactionType", escapeXML(firstDataTransactionType));
		parameters.put("testMode", escapeXML(firstDataTestMode));

		// add 'orderoptions to order
		order.addPart("orderoptions", op);

		// Build 'merchantinfo'
		op.clear();
		op.put("configfile", escapeXML(firstDataStoreNumber));

		// add 'merchantinfo to order
		order.addPart("merchantinfo", op);

		// Build 'transactiondetails'
		op.clear();

		// The order ID
		op.put("oid", escapeXML(Integer.toString(userOrder.getOrderID())));

		// Eci for Email or internet orders
		op.put("transactionorigin", "Eci");

		// If there is no PO Number for this order, pass a department code or
		// other value, but make sure the value you pass is supplied by the
		// customer
		// op.put("ponumber", "09876543-Q1234");

		// If the purchase is tax exempt, pass a value of Y for taxexempt
		op.put("taxexempt", "N");

		// Use UNSPECIFIED for ecommerce
		op.put("terminaltype", "UNSPECIFIED");

		// The customer's IP address
		op.put("ip", escapeXML((String) parameters.get("ipAddress")));

		// add 'billing to order
		order.addPart("transactiondetails", op);

		// Build 'billing' Required for AVS. If not provided, transactions will
		// downgrade.
		op.clear();
		if (StringUtils.isNotBlank(userOrder.getPostalCode())) {
			op.put("zip", escapeXML(userOrder.getPostalCode()));
		}

		// Derive the numerical part of the street address. Take the first term
		// of the address and strip non-digits.
		if (StringUtils.isNotBlank(userOrder.getAddress1())) {
			String addrnum = userOrder.getAddress1();
			String[] parts = addrnum.trim().split("\\s");
			if (parts.length > 0) {
				addrnum = parts[0];
			}
			addrnum = addrnum.replaceAll("\\D", "");
			op.put("addrnum", escapeXML(addrnum));
		}
		if (StringUtils.isNotBlank(userOrder.getFirstName())
				&& StringUtils.isNotBlank(userOrder.getLastName())) {
			op.put("name",
					escapeXML(userOrder.getFirstName() + " "
							+ userOrder.getLastName()));
		}
		if (StringUtils.isNotBlank(userOrder.getOrganization())) {
			op.put("company", escapeXML(userOrder.getOrganization()));
		}
		if (StringUtils.isNotBlank(userOrder.getAddress1())) {
			op.put("address1", escapeXML(userOrder.getAddress1()));
		}
		if (StringUtils.isNotBlank(userOrder.getAddress2())) {
			op.put("address2", escapeXML(userOrder.getAddress2()));
		}
		if (StringUtils.isNotBlank(userOrder.getCity())) {
			op.put("city", escapeXML(userOrder.getCity()));
		}
		if (StringUtils.isNotBlank(userOrder.getState())) {
			op.put("state", escapeXML(userOrder.getState()));
		}
		if (StringUtils.isNotBlank(userOrder.getCountry())) {
			op.put("country", escapeXML(userOrder.getCountry()));
		}

		// Strip out non-digits in the phone number
		if (StringUtils.isNotBlank(userOrder.getPhone1())) {
			op.put("phone",
					escapeXML(userOrder.getPhone1().replaceAll("\\D", "")));
		}
		// op.put("fax", "8059876543");
		if (StringUtils.isNotBlank(userOrder.getEmail1())) {
			op.put("email", escapeXML(userOrder.getEmail1()));
		}
		if (userOrder.getCustomerID() != null
				&& userOrder.getCustomerID().intValue() > 0) {
			op.put("userid", escapeXML(userOrder.getCustomerID().toString()));
		}
		// add 'billing to order
		order.addPart("billing", op);

		// build shipping
		op.clear();
		if (StringUtils.isNotBlank(userOrder.firstDelivery().getPostalCode())) {
			op.put("zip", escapeXML(userOrder.firstDelivery().getPostalCode()));
		}
		if (StringUtils.isNotBlank(userOrder.firstDelivery().getFirstName())
				&& StringUtils.isNotBlank(userOrder.firstDelivery()
						.getLastName())) {
			op.put("name", escapeXML(userOrder.firstDelivery().getFirstName()
					+ " " + userOrder.firstDelivery().getLastName()));
		}
		// if
		// (StringUtils.isNotBlank(userOrder.firstDelivery().getOrganization()))
		// {
		// op.put("company", escapeXML(userOrder.firstDelivery()
		// .getOrganization()));
		// }
		if (StringUtils.isNotBlank(userOrder.firstDelivery().getAddress1())) {
			op.put("address1", escapeXML(userOrder.firstDelivery()
					.getAddress1()));
		}
		if (StringUtils.isNotBlank(userOrder.firstDelivery().getAddress2())) {
			op.put("address2", escapeXML(userOrder.firstDelivery()
					.getAddress2()));
		}
		if (StringUtils.isNotBlank(userOrder.firstDelivery().getCity())) {
			op.put("city", escapeXML(userOrder.firstDelivery().getCity()));
		}
		if (StringUtils.isNotBlank(userOrder.firstDelivery().getState())) {
			op.put("state", escapeXML(userOrder.firstDelivery().getState()));
		}
		if (StringUtils.isNotBlank(userOrder.firstDelivery().getCountry())) {
			op.put("country", escapeXML(userOrder.firstDelivery().getCountry()));
		}
		order.addPart("shipping", op);

		// Build 'creditcard'
		op.clear();
		op.put("cardnumber",
				escapeXML((String) parameters.get("decryptedNumber")));
		op.put("cardexpmonth",
				escapeXML((String) parameters.get("creditCardExpirationMonth")));
		String creditCardExpirationYear = (String) parameters
				.get("creditCardExpirationYear");
		if (creditCardExpirationYear.length() == 4) {
			creditCardExpirationYear = creditCardExpirationYear.substring(2, 4);
		}
		op.put("cardexpyear", escapeXML(creditCardExpirationYear));
		String creditCardSecurityCode = (String) parameters
				.get("creditCardSecurityCode");
		if (StringUtils.isNotBlank(creditCardSecurityCode)) {
			op.put("cvmvalue", escapeXML(creditCardSecurityCode));
			op.put("cvmindicator", "provided");
		} else {
			op.put("cvmindicator", "not_provided");
		}

		// add 'creditcard to order
		order.addPart("creditcard", op);

		// Build 'payment'
		op.clear();
		op.put("subtotal", escapeXML(roundForMoney(userOrder.getSubtotal())
				.toString()));
		op.put("tax", escapeXML(roundForMoney(userOrder.getTax()).toString()));
		op.put("shipping", escapeXML(roundForMoney(userOrder.getShipping())
				.toString()));
		// op.put("vattax", "0.00");
		op.put("chargetotal", escapeXML(roundForMoney(userOrder.getTotal())
				.toString()));
		parameters.put("amount", roundForMoney(userOrder.getTotal()));

		// add 'payment to order
		order.addPart("payment", op);

		// add notes to order
		op.clear();
		// op.put("referred", "Saw ad on Web site.");
		op.put("comments", escapeXML((String) parameters.get("comments")));
		order.addPart("notes", op);

		String xmlRequest = order.toXML();

		if (log.isInfoEnabled()) {
			log.info("First Data XML request: "
					+ xmlRequest.replaceFirst("<cardnumber>.*?</cardnumber>",
							"").replaceFirst("<cvmvalue>.*?</cvmvalue>", ""));
		}

		return xmlRequest;
	}

	public Double roundForMoney(double value) {
		value = value * 100;
		value = Math.round(value);
		value = value / 100;
		return new Double(value);
	}

	public Double roundForMoney(Double value) {
		if (value != null) {
			return roundForMoney(value.doubleValue());
		} else {
			return new Double(0);
		}
	}

	public String escapeXML(String text) {
		if (text == null) {
			return "";
		}
		text = text.replaceAll("&", "&amp;");
		text = text.replaceAll("'", "&apos;");
		text = text.replaceAll("\"", "&quot;");
		text = text.replaceAll(">", "&gt;");
		text = text.replaceAll("<", "&lt;");
		return text;
	}

	public Map processResponse(String responseData, Map parameters)
			throws Exception {

		// Log the raw response and add it to the parameters for later storage
		parameters.put("responseData", responseData);
		if (log.isInfoEnabled()) {
			log.info("First Data response: " + responseData);
		}

		// Parse the response and log the parsed response as well
		Map responseMap = parseResponse(responseData);
		if (log.isDebugEnabled())
			log.info("First Data response map: " + responseMap);

		// Process the response.
		if (responseMap.get("r_error") != null
				&& !responseMap.get("r_error").equals("")) {
			log.info("Payment Error");
			return processErrorResponse(parameters, responseMap);
		}
		log.info("Payment Success");
		return processSuccessResponse(parameters, responseMap);
	}

	public Map parseResponse(String responseData) {
		Map responseMap = new HashMap();
		responseMap.put("r_time", parseTag("r_time", responseData));
		responseMap.put("r_ref", parseTag("r_ref", responseData));
		responseMap.put("r_approved", parseTag("r_approved", responseData));
		responseMap.put("r_code", parseTag("r_code", responseData));
		responseMap.put("r_authresronse",
				parseTag("r_authresronse", responseData));
		responseMap.put("r_error", parseTag("r_error", responseData));
		responseMap.put("r_ordernum", parseTag("r_ordernum", responseData));
		responseMap.put("r_message", parseTag("r_message", responseData));
		responseMap.put("r_score", parseTag("r_score", responseData));
		responseMap.put("r_tdate", parseTag("r_tdate", responseData));
		responseMap.put("r_avs", parseTag("r_avs", responseData));
		responseMap.put("r_tax", parseTag("r_tax", responseData));
		responseMap.put("r_shipping", parseTag("r_shipping", responseData));
		responseMap.put("r_fraudCode", parseTag("r_fraudCode", responseData));
		responseMap.put("esd", parseTag("esd", responseData));
		return responseMap;
	}

	public String parseTag(String tag, String rsp) {
		StringBuffer sb = new StringBuffer(256);
		sb.append('<' + tag + '>');
		int len = sb.length();
		int idxSt = -1, idxEnd = -1;
		if (-1 == (idxSt = rsp.indexOf(sb.toString()))) {
			return "";
		}
		idxSt += len;
		sb.setLength(0);
		sb.append("</" + tag + '>');
		if (-1 == (idxEnd = rsp.indexOf(sb.toString(), idxSt))) {
			return "";
		}
		return rsp.substring(idxSt, idxEnd);
	}

	public Map processErrorResponse(Map parameters, Map responseMap)
			throws Exception {
		Map result = new HashMap();
		String errorMessage = (String) responseMap.get("r_error");

		// Let's hide the location of the certificate key file from customers.
		if (errorMessage.startsWith("java.io.FileNotFoundException")) {
			result.put("resultCode", "10000");
			result.put("resultMessage", "Certificate key not found. "
					+ "Please contact the store administrator.");
			return result;
		}

		// Nicer message for declines
		if (responseMap.get("r_message") != null
				&& !responseMap.get("r_message").equals("DECLINED")) {
			result.put("resultCode", "10001");
			result.put("resultMessage", "Your card was declined.");
			return result;
		}

		// Most errors, send along the error message from First Data
		result.put("resultCode", "1");
		result.put("resultMessage", responseMap.get("r_error"));
		return result;
	}

	public Map processSuccessResponse(Map parameters, Map responseMap)
			throws Exception {
		Map result = new HashMap();
		Payment payment = (Payment) getBusinessObjectFactory().createObject(
				"paymentImplementer");
		BeanUtils.copyProperties(payment, parameters);
		payment.setStatus("Received");
		payment.setCreated(formatDateTime(new java.util.Date()));
		payment.setProcessorClassName(this.getClass().getName());
		String creditCardNumberDisplay = null;
		String decryptedNumber = payment.getDecryptedNumber();
		if (decryptedNumber != null) {
			creditCardNumberDisplay = "XXXXXXXXXXXX"
					+ decryptedNumber.substring(decryptedNumber.length() - 4,
							decryptedNumber.length());
			payment.setCreditCardNumberDisplay(creditCardNumberDisplay);
		}
		payment.setValueFieldLabels("TRANSACTION_TYPE,FIRST_DATA_TIMESTAMP,APPROVED,MESSAGE,AVS_CODE,APPROVAL_CODE,ORDER_NUMBER,REFERENCE_NUMBER,RAW_RESPONSE");
		payment.setValue1((String) parameters.get("transactionType"));
		payment.setValue2((String) responseMap.get("r_tdate"));
		payment.setValue3((String) responseMap.get("r_approved"));
		payment.setValue4((String) responseMap.get("r_message"));
		payment.setValue5((String) responseMap.get("r_avs"));
		payment.setValue6((String) responseMap.get("r_code"));
		payment.setValue7((String) responseMap.get("r_ordernum"));
		payment.setValue8((String) responseMap.get("r_ref"));
		payment.setValue9((String) parameters.get("responseData"));
		payment.setOrder(getUser().getOrder());
		if (getUser().getOrder().getPayments() == null)
			getUser().getOrder().setPayments(new LinkedHashSet());
		getUser().getOrder().getPayments().add(payment);
		PaymentDAO paymentDAO = (PaymentDAO) getDaoFactory().createDAO(
				"paymentDAOImplementer");
		paymentDAO.setPayment(payment);
		paymentDAO.insertPayment(false);
		result.put("newPayment", payment);
		result.put("resultCode", "0");
		result.put("resultMessage", responseMap.get("r_message"));
		return result;
	}
}
