package com.softslate.commerce.customer.product;

import java.util.Map;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.commons.beanutils.BeanUtils;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.struts.action.ActionForm;
import org.apache.struts.action.ActionForward;
import org.apache.struts.action.ActionMapping;

import com.softslate.commerce.businessobjects.product.LuceneSearcher;
import com.softslate.commerce.businessobjects.product.ProductList;
import com.softslate.commerce.businessobjects.product.ProductListBean;
import com.softslate.commerce.businessobjects.product.ProductProcessor;
import com.softslate.commerce.businessobjects.product.SearchList;
import com.softslate.commerce.businessobjects.product.SearchListBean;
import com.softslate.commerce.customer.core.BaseAction;
import com.softslate.commerce.customer.core.BaseDynaForm;

/**
 * Struts action class that corresponds to the product list page. Used for
 * requests to "/ProductList.do".
 * <p>
 * The <code>execute</code> method of this class retrieves information from the
 * database for the list of active products in the system and puts it in the
 * request scope for use by the view layer. Given a
 * {@link com.softslate.commerce.customer.product.ProductListForm}, which may
 * have its <code>firstRow</code> property populated for pagination, it creates
 * and initializes an instance of
 * {@link com.softslate.commerce.businessobjects.product.ProductList}. It then
 * invokes
 * {@link com.softslate.commerce.businessobjects.product.ProductProcessor#getProductList(ProductList)}
 * to load the product list information from the database. It puts the
 * initialized <code>ProductList</code> object in the request scope as an
 * attribute with the name "productList".
 * 
 * @author David Tobey
 * 
 * @see com.softslate.commerce.customer.product.ProductListForm
 * @see com.softslate.commerce.businessobjects.product.ProductList
 * @see com.softslate.commerce.businessobjects.product.ProductProcessor
 */
public class ProductListAction extends BaseAction {

	static Log log = LogFactory.getLog(ProductListAction.class);

	public ActionForward execute(ActionMapping mapping, ActionForm form, HttpServletRequest request,
			HttpServletResponse response) throws Exception {
		if (log.isInfoEnabled())
			log.info("Starting action.");
		BaseDynaForm baseForm = (BaseDynaForm) form;
		baseForm.initializeProperties(mapping, request);
		String itemsPerPage = determineItemsPerPageAsHandleViewAll(baseForm);

		// If advanced search is on, load the search list using Lucene
		String advancedSearchProductList = (String) baseForm.getSettings().getValue("advancedSearchProductList");
		request.setAttribute("linkBase", "/ProductList.do?");
		if (advancedSearchProductList.equals("1")) {
			SearchList searchList = loadProductsWithAdvancedSearch(baseForm, itemsPerPage,
					(String) request.getParameter("sortBy"));
			request.setAttribute("searchList", searchList);
			request.setAttribute("advancedSearch", "true");
			request.setAttribute("itemCount", new Integer(searchList.getItemCount()));
			request.setAttribute("itemsPerPage", new Integer(searchList.getItemsPerPage()));
			request.setAttribute("firstRow", new Integer(searchList.getFirstRow()));
			request.setAttribute("listCollection", searchList.getSearchListCollection());
			request.setAttribute("objectTypes", searchList.getObjectTypes());
			String sort = searchList.getSortBy();
			if (!"score".equals(sort)) {
				request.setAttribute("sortByParam", "&sortBy=" + java.net.URLEncoder.encode(sort, "UTF-8"));
			}
			String facetsParam = baseForm.getFormUtils().stringArrayAsURLParam("facets", searchList.getFacets());
			request.setAttribute("facetsParam", facetsParam);
		} else {
			ProductList productList = loadProductListFromDatabase(baseForm, itemsPerPage);
			request.setAttribute("productList", productList);
			request.setAttribute("itemCount", new Integer(productList.getItemCount()));
			request.setAttribute("itemsPerPage", new Integer(productList.getItemsPerPage()));
			request.setAttribute("firstRow", new Integer(productList.getFirstRow()));
			request.setAttribute("listCollection", productList.getProductListCollection());
		}

		// Forward to Ajax tile if it's an Ajax request (faceted search)
		if (baseForm.getMap().get("ajax") != null && baseForm.getMap().get("ajax").equals("true")) {
			if (("default-html5").equals(baseForm.html5OrXhtml())) {
				return mapping.findForward("successAjaxHTML5");
			} else {
				return mapping.findForward("successAjax");
			}
		}

		return mapping.findForward("success");
	}

	protected String determineItemsPerPageAsHandleViewAll(BaseDynaForm baseForm) {
		String itemsPerPage = (String) baseForm.getSettings().getValue("productListPageSize");
		if (baseForm.getMap().get("firstRow") != null && baseForm.getMap().get("firstRow").equals("all")) {
			itemsPerPage = (String) baseForm.getSettings().getValue("maxRecordsProcessed");
			baseForm.getMap().put("firstRow", "1");
		}
		return itemsPerPage;
	}

	protected SearchList loadProductsWithAdvancedSearch(BaseDynaForm baseForm, String itemsPerPage, String sortByParam)
			throws Exception {

		// First create the processor to do the searching
		LuceneSearcher luceneSearcher = (LuceneSearcher) baseForm.getBusinessObjectFactory().createObject(
				"luceneSearcherImplementer");

		// Create a SearchList helper bean
		SearchList searchList =  baseForm.getBusinessObjectFactory().createObject(SearchList.class);
		BeanUtils.copyProperties(searchList, baseForm);
		baseForm.getFormUtils().eliminateDuplicateFacets(searchList);

		// Set the path to the index
		String index = (String) baseForm.getSettings().getValue("luceneIndexDir");
		searchList.setIndexDir(getServlet().getServletContext().getRealPath(index));
		searchList.setItemsPerPage(Integer.parseInt(itemsPerPage));
		searchList.setSearchListType(SearchListBean.SEARCH_PRODUCTS_FOR_PRODUCT_LIST);
		if (sortByParam != null) {
			searchList.setSortBy(sortByParam);
		} else {
			searchList.setSortBy("productOrder");
		}
		searchList.setCategoryMap((Map) getServlet().getServletContext().getAttribute("categoryMap"));

		// Load the objects and place the list in the request scope
		searchList = luceneSearcher.getSearchList(searchList);
		return searchList;
	}

	protected ProductList loadProductListFromDatabase(BaseDynaForm baseForm, String itemsPerPage) throws Exception {

		// Create a processor to use to load the products
		ProductProcessor productProcessor = (ProductProcessor) baseForm.getBusinessObjectFactory().createObject(
				"productProcessorImplementer");
		ProductList productList = (ProductList) baseForm.getBusinessObjectFactory().createObject(
				"productListImplementer");
		BeanUtils.copyProperties(productList, baseForm);
		if (log.isDebugEnabled())
			log.debug("itemsPerPage: " + itemsPerPage);
		productList.setItemsPerPage(Integer.parseInt(itemsPerPage));
		productList.setProductListType(ProductListBean.ALL_ACTIVE);

		// Load the products and place the list in the request scope
		productList = productProcessor.getProductList(productList);
		return productList;

	}
}
